package de.hft.stuttgart.citydoctor2.gui;

import java.io.IOException;
import java.io.InputStream;

import de.hft.stuttgart.citydoctor2.utils.Localization;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.stage.Window;

public class AboutDialog {
	
	@FXML
	private ImageView logoView;

	@FXML
	private ImageView beuthView;

	@FXML
	private ImageView minisView;

	@FXML
	private ImageView hftView;

	@FXML
	private Button closeBtn;
	
	@FXML
	private Label cdLabel;

	private Stage stage;

	public AboutDialog(Window parent) throws IOException {
		FXMLLoader loader = new FXMLLoader(AboutDialog.class.getResource("AboutDialog.fxml"));
		loader.setController(this);
		VBox box = loader.load();
		

		stage = new Stage(StageStyle.UTILITY);
		stage.setTitle(Localization.getText("AboutDialog.title"));
		stage.setResizable(false);
		Scene scene = new Scene(box);
		stage.setScene(scene);
		stage.initOwner(parent);
		stage.initModality(Modality.APPLICATION_MODAL);
	}

	public void initialize() {
		String developedBy = Localization.getText("AboutDialog.developedBy");
		String contact = Localization.getText("AboutDialog.contact");
		closeBtn.setText(Localization.getText("AboutDialog.closeBtn"));
		cdLabel.setText("CityDoctor Version " + Localization.getText(Localization.VERSION) + "\n\n" + 
				developedBy + ":\n" + 
				"Hochschule für Technik\n" + 
				"Beuth Hochschule Berlin\n\n" +
				contact + ":\n" + 
				"Matthias Betz\n" + 
				"matthias.betz@hft-stuttgart.de");

		
		closeBtn.setOnAction(ae -> hide());
		try {
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/CityDoctor-Logo-rot_klein.png")) {
				Image img = new Image(inStream);
				logoView.setImage(img);
			}
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/hft_logo.png")) {
				Image img = new Image(inStream);
				hftView.setImage(img);
			}
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/Beuth-Logo_basis.png")) {
				Image img = new Image(inStream);
				beuthView.setImage(img);
			}
			try (InputStream inStream = MainWindow.class.getResourceAsStream("icons/BMBF_Logo.png")) {
				Image img = new Image(inStream);
				minisView.setImage(img);
			}
		} catch (IOException e) {
			// ignore close exception
		}
	}
	
	public void show() {
		stage.show();
	}

	public void hide() {
		stage.hide();
	}

}
